
function [xistar, xi0, Istar, Loss, mu0, finalloss, finalloss0] = ...
    eg1(N, nu, n0, delta0)

format short g
close ALL

% eg: [xistar, xi0, Istar, Loss, mu0, finalloss] = eg1(100, 1, 1, .002);

x = linspace(1/N, 1, N);
q = x'/norm(x);
p = q.^2;
indices = 1:N;

% Initial design: size n0, multinomial frequencies with randomly chosen
% probabilities. The default is a one point design.
initprob = rand(N,1);
initprob = initprob/sum(initprob);
xi = mnrnd(n0, initprob)'/n0;

r = sum(xi.*p);
s = sum((xi.^2).*p);
designsize = n0;
Istar = indices(xi>0); %%Contains the indices of design points, in the order 
    % in which they enter the design
r = sum(xi.*p);
s = sum((xi.^2).*p);
Loss = (1 - nu)/r + nu*s/r^2;
delta = 1; % delta0 (set as input) controls the stopping rule

%% run the iterations: choose a new design point, recompute loss and other measures
if nu == 1
    ds = 10*N + n0;
else
    ds = 3;
end
while delta > delta0 || designsize < ds
    testvec = p.*((1-nu)*r + 2*nu*(s - r*xi));
    if nu < 1
        delta = max(testvec)/((1-nu)*r^2) - 1;
    else         
        delta = max(testvec);
    end
    [~, istar] = max(testvec);
    if length(istar) >1
        istar = istar(1);
    end
    xi = (designsize*xi + [zeros((istar-1),1); 1; zeros(N-istar, 1)])/(designsize + 1);
    designsize = designsize + 1;
    r = sum(xi.*p);
    s = sum((xi.^2).*p);
    loss = (1 - nu)/r + nu*s/r^2;
    Istar(designsize-n0) = istar;
    Loss(designsize-n0) = loss;
end
% Delete the initial sample; 
% form the final design xistar from Istar alone
%nfinal = designsize-n0; %final size of design
% Delete the first 100 points
Istar = Istar(101:length(Istar));
nfinal = length(Istar);
%length(Istar)
%pause
tab = tabulate(Istar);
xistar = tab(:,2)/nfinal;
%sum(xistar) % check that this = 1
r = sum(xistar.*p);
s = sum((xistar.^2).*p);
finalloss = (1 - nu)/r + nu*s/r^2;
testvec = (p.*((1-nu)*r + 2*nu*(s - r*xistar)));

if nu < 1
    delta = max(testvec)/((1-nu)*r^2) - 1
else
    delta = max(testvec)
end

% Find minimax design xi0 and minimum possible loss:
func = @(mu) minloss(mu, nu, p);
[mu0, L0] = fminbnd(func, 0, max(q));
num0 = max( 1-(mu0^2)./p, 0);
xi0 = num0/sum(num0);
r0 = sum(xi0.*p);
s0 = sum((xi0.^2).*p);
finalloss0 = (1 - nu)/r0 + nu*s0/r0^2;
testvec0 = (p.*((1-nu)*r0 + 2*nu*(s0 - r0*xi0)));

if nu < 1
    delta0 = max(testvec0)/((1-nu)*r^2) - 1
else
    delta0 = max(testvec0)
end
nstar = length(Istar)

disp(['loss of xi0 = ', num2str(finalloss0), ' loss of xistar = ', num2str(finalloss)]);
% Do the plots
subplot(2,2,1)
hold on
xlim([1 nfinal]);
plot((2:nfinal), x(Istar(2:nfinal)))
xlabel('(a)')
ylabel('x_{i^{*}}')

subplot(2,2,2)
str = {['Final Loss = ', num2str(Loss(length(Loss)))]};
hold on
xlim([0,nfinal + 5]);
ylim([L0-.01 min(Loss)+.1]);
plot(Loss(2:nfinal))
plot([0 nfinal], [L0 L0]);
xlabel('(b)')
ylabel('$\mathcal{I}_{\nu}(\xi_{n}^{*})$','Interpreter','latex')
if nu == .5 
    textheight = L0 + .6;
else 
    textheight = L0 + .8;
end
%text(.05*nfinal, textheight, str, 'HorizontalAlignment', 'left','Interpreter','latex')
hold off

if nu == 1
    textheight = 1.3*max(xi0);
else
    textheight = .9*max(xi0);
end
barwidth = .01;

subplot(2,2,3)
str1 = {['$\mathcal{I}_{\nu}(\xi_{n^{*}}^{*})$ = ', num2str(round(finalloss,4))]};
hold on
xlim([-.05,1.05]);
ylim([0 1.2*textheight]);
bar(x, xistar, barwidth);
xlabel('(c)')
ylabel('\xi_{n^{*}}^{*}(x)')
text(.05,textheight, str1,'Interpreter','latex')
hold off

subplot(2,2,4)
str2 = {['$\mathcal{I}_{\nu}(\xi^{0})$ = ', num2str(round(L0,4))]};
hold on
xlim([-.05,1.05]);
ylim([0 1.2*textheight]);
bar(x, xi0, barwidth)
xlabel('(d)')
ylabel('\xi^0(x)')
text(.05,textheight, str2,'Interpreter', 'latex');
hold off

%save slr


%% ============================= subfunctions =============================
%% ============================= minloss =============================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out = minloss(mu, nu, p) % Constructs the minimax (static) design
num = max( 1-(mu^2)./p, 0);
xi = num/sum(num);
r = sum(xi.*p);
out = (1-nu)/r + nu*sum((xi.^2).*p)/r^2;



