
function [alpha, beta, Lambda, Xi0, Xi0exact, Xistar, Xistarexact,OUTMAT] ...
    = eg3(N2, nu, n0, delta0, n, criterion, inputPso)

format short g
close ALL
tstart = tic;

% n is size of desired exact design
% criterion = 'I' or 'D'
% Put inputPso = 1 if I have good pso output already
% [alpha, beta, Lambda, Xi0, Xi0exact, Xistar, Xistarexact, OUTMAT] = eg3(50, .5, 12, .01, 50, 'I', 1);

%% Housekeeping:
[N, X, x1, ~, ~, Q] = eg3data(N2);

%% Compute the parametric design

[alpha, beta, Lambda, xi0loss, xi0] = eg3constrained(criterion, nu, N2, Q, inputPso);
xi0exact = xi0;
xi0exact(xi0 < 1/n) = 0; % remove small weights, then renormalize
xi0exact = xi0exact/sum(xi0exact);

% Do the rounding, obtain the exact design of size n
indices = (1:N)';
[supportindices, frequency, ~] = allocate(n, xi0exact, indices);
xi0exact = zeros(N,1);
xi0exact(supportindices) = frequency/n;
[xi0exactloss, ~, ~]  = summary(xi0exact, nu, Q, criterion);
% Final output:
Xi0 = [X(xi0>0,:) xi0(xi0>0)];
Xi0exact = [X(xi0exact>0,:) xi0exact(xi0exact>0)];

%% Compute the sequential design
Istar = [];
Loss = [];
% Initial design randomly chosen: size n0
xi = zeros(N,1);
xi(randsample(N,n0)) = 1;
xi = xi/sum(xi);
[~, Inext, ~]  = summary(xi, nu, Q, criterion); % current loss and next design point
designsize = n0;
delta = 1;

%% Do the iterations:
while delta > delta0 
    istar = Inext;
    xi = (designsize*xi + [zeros((istar-1),1); 1; zeros(N-istar, 1)])/(designsize + 1);
    designsize = designsize + 1;
    [Lnow, Inext, delta]  = summary(xi, nu, Q, criterion);
    if rem(designsize, 100) == 0
        disp(['design size = ', num2str(designsize), ...
            ' i* = ', num2str(istar), ' delta = ', num2str(delta)]);
    end
    Istar(designsize-n0) = istar;
    Loss(designsize-n0) = Lnow;
end

% Delete the initial sample; 
% form the final design xistar from Istar alone

nfinal = designsize - n0;
tab = tabulate(Istar);
xistar = tab(:,2)/nfinal;
[xistarloss, ~, ~]  = summary(xistar, nu, Q, criterion);

xistarexact = xistar;
 %%%%%%%%%NOTE%%%%%%%%%%%%
xistarexact(xistar < 1/n) = 0; % remove small weights, then renormalize
xistarexact = xistarexact/sum(xistarexact);

% Do the rounding, obtain the exact design of size n
indices = (1:N)';
[supportindices, frequency, ~] = allocate(n, xistarexact, indices);
xistarexact = zeros(N,1);
xistarexact(supportindices) = frequency/n;
[xistarexactloss, ~, ~]  = summary(xistarexact, nu, Q, criterion);


%% Final output:
Xistar = [X(xistar>0,:) xistar(xistar>0)];
Xistarexact = [X(xistarexact>0,:) xistarexact(xistarexact>0)];

OUTMAT = [xi0loss xistarloss; size(Xi0,1), size(Xistar,1); ...
        xi0exactloss xistarexactloss; size(Xi0exact,1), size(Xistarexact,1)];
RLAB =  'loss(xi) size(xi) loss(xiexct) size(xiexct)';
CLAB = 'parametric sequential';

disp(' ');
disp(' ');
disp(['nu = ', num2str(nu), ' criterion = ', criterion]);
printmat(OUTMAT, 'Comparisons', RLAB, CLAB);
    
telapsed = toc(tstart);

%% Plotting

figure(1)
subplot(1,2,1)
view(0, 0);
%zlim([0 .1]);
ylim([0 8]);
C = Xi0(:,1);
C(Xi0(:,1) == x1(1)) = 'r';
C(Xi0(:,1) == x1(2)) = 'b';
C(Xi0(:,1) == x1(3)) = 'g';
C(Xi0(:,1) == x1(4)) = 'k';
scatter3(Xi0(:,1), Xi0(:,2), Xi0(:,3), [], C, 'filled');
xlabel('x1');
ylabel('x2');
zlabel('$\xi^{0}$','Interpreter','latex');


subplot(1,2,2)
ylim([0 8]);
C = Xi0exact(:,1);
C(Xi0exact(:,1) == x1(1)) = 'r';
C(Xi0exact(:,1) == x1(2)) = 'b';
C(Xi0exact(:,1) == x1(3)) = 'g';
C(Xi0exact(:,1) == x1(4)) = 'k';
scatter3(Xi0exact(:,1), Xi0exact(:,2), Xi0exact(:,3), [], C, 'filled');
xlabel('x1');
ylabel('x2');
zlabel('$\xi^{0}_{exact}$','Interpreter','latex');

figure(2)
subplot(1,2,1)
view(0, 0);
ylim([0 8]);
C = Xistar(:,1);
C(Xistar(:,1) == x1(1)) = 'r';
C(Xistar(:,1) == x1(2)) = 'b';
C(Xistar(:,1) == x1(3)) = 'g';
C(Xistar(:,1) == x1(4)) = 'k';
scatter3(Xistar(:,1), Xistar(:,2), Xistar(:,3), [], C, 'filled');
xlabel('x1');
ylabel('x2');
zlabel('$\xi^{*}$','Interpreter','latex');


subplot(1,2,2)
ylim([0 8]);
C = Xistarexact(:,1);
C(Xistarexact(:,1) == x1(1)) = 'r';
C(Xistarexact(:,1) == x1(2)) = 'b';
C(Xistarexact(:,1) == x1(3)) = 'g';
C(Xistarexact(:,1) == x1(4)) = 'k';
scatter3(Xistarexact(:,1), Xistarexact(:,2), Xistarexact(:,3), [], C, 'filled');
xlabel('x1');
ylabel('x2');
zlabel('$\xi^{*}_{exact}$','Interpreter','latex');

telapsed

%% Test the necessary condition

delta
[~, ~, delta1]  = summary(xistar, nu, Q, criterion);
disp(['Final design size = ', num2str(designsize - n0), ...
    ' delta for xistar = ', num2str(delta1)]);
if criterion == 'I'
    if nu == 0
        save lubricantzero_I
    else
        save lubricantpositive_I
    end
elseif criterion == 'D'
    if nu == 0
        save lubricantzero_D
    else
        save lubricantpositive_D
    end
end




