function [alpha, beta, Lambda, loss] = eg3pso(criterion, nu, N2)

% Uses particle swarm optimization to get starting values for eg3constrained
format short g
close ALL
%tstart = tic;

% n is size of desired exact design
% criterion = 'I' or 'D'


% [alpha, beta, Lambda, loss] = eg3pso('I', .5, 50)
[~, ~, ~, ~, ~, Q] = eg3data(N2);
p = size(Q,2);

fun = @(theta) lossonly(theta, Q, criterion, nu, p);
options = optimoptions('particleswarm', 'Display', 'iter', 'HybridFcn', @fminunc);
%, 'HybridFcn', @fminunc
thetax = particleswarm(fun, 54, [], [], options);

% prepare final output
[thetax, ~, loss] = phi_search(thetax, Q, criterion, nu, p);
disp(['final loss using pso = ', num2str(loss)]);

rounder = 4;
alpha = round(thetax(1), rounder);
beta = round(thetax(2:p), rounder);
vec = thetax((p+1):length(thetax));
Lambda = round(unvecs(vec,p), rounder);
if criterion == 'I'
    if nu == 0
        savefile = 'eg3pso_zero_I';
        save(savefile, 'alpha', 'beta', 'Lambda');
    else
        savefile = 'eg3pso_pos_I';
        save(savefile, 'alpha', 'beta', 'Lambda');
    end
elseif criterion == 'D'
    if nu == 0
        savefile = 'eg3pso_zero_D';
        save(savefile, 'alpha', 'beta', 'Lambda');
    else
        savefile = 'eg3pso_pos_D';
        save(savefile, 'alpha', 'beta', 'Lambda');
    end
end

%% ============================= subfunction "phi_search" =============================
% accepts a vector of parameters, returns (i) parameters meeting the
% constraints, (ii) design, (iii) loss, both evaluated at the new
% parameters
function [newtheta, xi, loss] = phi_search(theta, Q, criterion, nu, p)
alpha = theta(1);
beta = theta(2:p);
betaALL = [1; beta'];
vec = theta((p+1):length(theta));
Lambda = unvecs(vec,p);

xi = max(alpha + diag(Q*Lambda*Q'),0)./diag(Q*(betaALL*betaALL')*Q');

if all(xi==0)
    loss = 1e6;
    theta = theta';
    newtheta = theta + rand*ones(length(theta),1);
    N = length(xi);
    xi = ones(N,1)/N;
else
    I = sum(xi);
    betaALL = betaALL*sqrt(I); % now sum(xi) = 1
    Lambda = Lambda/(betaALL(1)^2);
    alpha = alpha/(betaALL(1)^2);
    betaALL = betaALL/betaALL(1); % now betaALL(1) = 1
    xi = max(alpha + diag(Q*Lambda*Q'),0)./diag(Q*(betaALL*betaALL')*Q');
    newtheta = [alpha;betaALL(2:p); vecs(Lambda, p)];    
    [loss, ~, ~] = summary(xi, nu, Q, criterion);
end

%% ============================= subfunction "lossonly" =============================
function Loss = lossonly(theta, Q, criterion, nu, p)
[~,~, Loss] = phi_search(theta, Q, criterion, nu, p);

%% ============================= subfunction "vecs" =============================
%Forms a vector from the lower triangle of a matrix
function vec = vecs(mat, p)
vec = [];
for j = 1:p
    vec = [vec; mat(j:p,j)];
end

% ============================= subfunction "unvecs" =============================
%Forms a matrix from the output of vecs
function mat = unvecs(vec, p)
mat = zeros(p);
for j = 1:p
    last = (j-1)*(2*p - j + 2)/2;
    mat(j:p,j) = vec((last + 1):(last + (p - j + 1)));
end
mat = mat + mat';
mat = mat - diag(diag(mat))/2;



