function [alpha, beta, Lambda, xi0, loss0] = parametric(criterion, Q,  nu)

format short g

% criterion = 'I' or 'D'
p=2;

%% Housekeeping:

%alpha0 = randn; %.003;
%beta0 = [.9; -.4];
%Lambda0 = [4 8; 8 3];%[-.04 -.08; -.08 .03];

load bestadaptive
alpha0 = alpha; % + randn
beta0 = beta;
Lambda0 = Lambda;

theta0 = [alpha0; beta0; vecs(Lambda0, p)];
[theta0, ~, ~] = phi(theta0, Q, criterion, nu, p);

fun = @(theta) lossonly(theta, Q, criterion, nu, p);
nonlcon = @(theta) phicon(theta, Q, p);

options = optimset('Display', 'final', 'algorithm', 'interior-point',...
    'TolFun', 1e-8, 'TolX', 1e-8, 'TolCon', 1e-8, ...
    'MaxIter', 1000, 'MaxFunEvals', 10000);
% Display 'iter' or 'final'
% try 'interior-point' (default), then 'sqp', then'active-set' algorithm
[theta, ~, exitflag, output] = ...
    fmincon(fun,theta0,[],[],[],[],[],[],nonlcon, options);
exitflag
output

% prepare plots and final output

[theta, xi0, loss0] = phi(theta, Q, criterion, nu, p);

rounder = 4;
alpha = round(theta(1), rounder);
beta = round(theta(2:(p+1)), rounder);
vec = theta((p+2):length(theta));
Lambda = round(unvecs(vec,p), rounder);

%% ============================= subfunction "phi" =============================
% accepts a vector of parameters, returns (i) parameters meeting the
% constraints, (ii) design, (iii) loss, both evaluated at the new
% parameters
function [newtheta, xi, loss] = phi(theta, Q, criterion, nu, p)
alpha = theta(1);
beta = theta(2:(p+1));
vec = theta((p+2):length(theta));
Lambda = unvecs(vec,p);

xi = max(alpha + diag(Q*Lambda*Q'),0)./diag(Q*(beta*beta')*Q');

if all(xi==0)
    loss = 1e6;
    newtheta = theta + rand*ones(length(theta),1);
    N = length(xi);
    xi = ones(N,1)/N;
else    
    I = sum(xi);
    beta = beta*sqrt(I); % now sum(xi) = 1 
    alpha = alpha/(norm(beta)^2);
    Lambda = Lambda/(norm(beta)^2);
    beta = beta/norm(beta); % now norm(beta) = 1
    xi = max(alpha + diag(Q*Lambda*Q'),0)./diag(Q*(beta*beta')*Q');
    newtheta = [alpha; beta; vecs(Lambda, p)];
    [loss, ~, ~, ~]  = eg4summary(xi, nu, Q, criterion);
end



%% ============================= subfunction "lossonly" =============================
function Loss = lossonly(theta, Q, criterion, nu, p)
[~,~, Loss] = phi(theta, Q, criterion, nu, p);

%% ============================= subfunction "phicon" =============================
function [c,ceq] = phicon(theta, Q, p)
c = [];
alpha = theta(1);
beta = theta(2:(p+1));
vec = theta((p+2):length(theta));
Lambda = unvecs(vec,p);
xi = max(alpha + diag(Q*Lambda*Q'),0)./diag(Q*(beta*beta')*Q');
ceq = [sum(xi) - 1; norm(beta) - 1];

%% ============================= subfunction "vecs" =============================
%Forms a vector from the lower triangle of a matrix
function vec = vecs(mat, p)
vec = [];
for j = 1:p
	vec = [vec; mat(j:p,j)];
end
        
% ============================= subfunction "unvecs" =============================
%Forms a matrix from the output of vecs
function mat = unvecs(vec, p)
mat = zeros(p);
    for j = 1:p
        last = (j-1)*(2*p - j + 2)/2;
        mat(j:p,j) = vec((last + 1):(last + (p - j + 1)));
    end
mat = mat + mat';
mat = mat - diag(diag(mat))/2;
                     

                        
