function [Lnow, Inext, delta]  = summary(xi, nu, Q, criterion) 
%% loss, next sequentially chosen obsevation and diagonal of the matrix T
D = diag(xi);
R = Q'*D*Q;
num = rcond(R);
p = size(R,1);
Ip = eye(p);
%Rinv = R\Ip;
%S = Q'*(D^2)*Q;
%U = Rinv*S*Rinv';
if num < 1e-5 || isnan(num)
    Lnow = 1e6;
    R = Ip;
    Rinv = R\Ip;
    S = Q'*(D^2)*Q;
    U = Rinv*S*Rinv';
else
    p = size(R,1);
    Ip = eye(p);
    Rinv = R\Ip;
    S = Q'*(D^2)*Q;
    U = Rinv*S*Rinv';
    if criterion == 'I'
        if nu > 0
            [~, lambda] = eigs(U,1);
        else
            lambda = 0;
        end
        Lnow = (1-nu)*trace(Rinv) + nu*lambda;
    elseif criterion == 'D'
        rootR = sqrtm(R);
        if nu > 0
            [~, lambda] = eigs(rootR*(U - Ip)*rootR', 1);
        else
            lambda = 0;
        end
        Lnow = ((1-nu + nu*lambda)/det(R))^(1/p);
    end
end
Tmat = T(nu, R, U, D, Q, criterion);
[d, Inext] = max(diag(Tmat));
delta = d/trace(D*Tmat) - 1;


%% ============================= subfunctions =============================
%% ============================= T =============================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out = T(nu, R, U, D, Q, criterion)

if criterion == 'I'
    Rinv = R\eye(size(R));
    [z, lambda] = eigs(U,1);
    vec = Rinv*z;
    B = 2*lambda*z*vec';
    C = 2*(vec*vec');
    mat = Q*Rinv;
    out = (1-nu)*(mat*mat') + nu*(Q*B*Q' - D*Q*C*Q');
elseif criterion == 'D'
    Ip = eye(size(R));
    rootR = sqrtm(R);
    rootRinv = rootR\Ip;
    Rinv = R\Ip;
    [z, lambda] = eigs(rootR*(U - Ip)*rootR', 1);
    v = rootR*z;
    w = rootRinv*z;
    J = lambda*(Rinv + (w*w')) + (w*v' + v*w');
    K = 2*(w*w');
    out = (1-nu)*Q*Rinv*Q' + nu*(Q*J*Q' - D*Q*K*Q');
end



